﻿"use strict";

var Audio = Audio || {};

Audio.AudioContext = window.AudioContext || window.webkitAudioContext;
Audio.context = new Audio.AudioContext();

Audio.f_musicVolume;
Audio.f_soundVolume;
Audio.i_noteCurrent;
Audio.i_noteLimit;
Audio.i_timer;

// INITIALIZE LOADING VARIABLES
Audio.a_sounds = [];
Audio.a_music = [];
Audio.i_content = 0;
Audio.i_contentLoaded = 0;

// AUDIO CONSTANTS
Audio.I_AUDIO_MLIMIT = 3;
Audio.I_AUDIO_MTIMER = 32;
Audio.F_AUDIO_SVOLUME = 1;
Audio.F_AUDIO_MVOLUME = 1;


Audio.fu_LoadSound = function (name) {

    var o_request = new XMLHttpRequest();
    o_request.open('GET', "Resources/Sounds/" + name + ".mp3", true);
    o_request.responseType = 'arraybuffer';

    // DECODE ASYNCHRONOUSLY
    o_request.onload = function () {
        Audio.context.decodeAudioData(o_request.response, function (buffer) {

            // LOAD THE SOUND EFFECT
            var id = Audio.a_sounds.length;
            Audio.a_sounds[id] = buffer;
            Audio.a_sounds[id].name = name;

            GameLoop.i_loaded++;

            console.log('LOADED SOUND ' + name);

            GameLoop.fu_ContentCheck();

        }, null);
    }

    GameLoop.i_loading++;
    o_request.send();

}

Audio.fu_LoadNote = function (name) {

    var o_request = new XMLHttpRequest();
    o_request.open('GET', "Resources/Notes/" + name + ".mp3", true);
    o_request.responseType = 'arraybuffer';

    // DECODE ASYNCHRONOUSLY
    o_request.onload = function () {
        Audio.context.decodeAudioData(o_request.response, function (buffer) {

            // LOAD THE SOUND EFFECT
            var id = Audio.a_music.length;
            Audio.a_music[id] = buffer;
            Audio.a_music[id].name = name;

            GameLoop.i_loaded++;

            GameLoop.fu_ContentCheck();

        }, null);
    }

    GameLoop.i_loading++;
    o_request.send();

}

Audio.fu_LoadContent = function () {

    Audio.fu_LoadNote("music1");
    Audio.fu_LoadNote("music2");
    Audio.fu_LoadNote("music3");
    Audio.fu_LoadNote("music4");
    Audio.fu_LoadSound("meow");


}

Audio.fu_Initialize = function () {
    Audio.i_noteCurrent = 0;
    Audio.i_noteLimit = Audio.I_AUDIO_MLIMIT;
    Audio.i_timer = 0;
    Audio.f_musicVolume = Audio.F_AUDIO_MVOLUME;
    Audio.f_soundVolume = Audio.F_AUDIO_SVOLUME;
}

Audio.fu_PlaySound = function (name, volume) {

    // SET DEFAULT VOLUME
    if (volume == null)
        volume = 1;

    Audio.fu_Clamp(volume, 0, 1);

    // CHECK THAT THE FILE EXISTS
    for (var i = 0; i < Audio.a_sounds.length; i++)
        if (name == Audio.a_sounds[i].name) {

            // SET THE SOURCE BUFFER
            var source = Audio.context.createBufferSource();
            source.buffer = Audio.a_sounds[i];

            // SET GAIN TO ADJUST VOLUME
            var sound_gain = Audio.context.createGain();
            sound_gain.gain.value = Audio.f_soundVolume;
            sound_gain.connect(Audio.context.destination);
            source.connect(sound_gain);

            // PLAY THE SOUND EFFECT
            source[source.start ? 'start' : 'noteOn'](0);

            // GTFO
            return;
        }
}

Audio.fu_PlayNote = function (name, volume) {

    // SET DEFAULT VOLUME
    if (volume == null)
        volume = 1;

    Audio.fu_Clamp(volume, 0, 1);

    // CHECK THAT THE FILE EXISTS
    for (var i = 0; i < Audio.a_music.length; i++)
        if (name == Audio.a_music[i].name) {

            // SET THE SOURCE BUFFER
            var source = Audio.context.createBufferSource();
            source.buffer = Audio.a_music[i];

            // SET GAIN TO ADJUST VOLUME
            var sound_gain = Audio.context.createGain();
            sound_gain.gain.value = Audio.f_soundVolume;
            sound_gain.connect(Audio.context.destination);
            source.connect(sound_gain);

            // PLAY THE SOUND EFFECT
            source[source.start ? 'start' : 'noteOn'](0);

            // GTFO
            return;
        }
}

Audio.fu_Update = function () {

    // ACTUATE BASED ON TIMER DECREMENTATION
    if (--Audio.i_timer <= 0) {

        // PLAY CURRENT MUSIC NOTE
        Audio.fu_PlayNote(Audio.a_music[Audio.i_noteCurrent].name);

        // RESET NOTE SEQUENCE OVERFLOW 
        if (Audio.i_noteCurrent++ >= Audio.i_noteLimit)
            Audio.i_noteCurrent = 0;

        // RESET TIMER
        Audio.i_timer = Audio.I_AUDIO_MTIMER;

    }
}

// CLAMP AUDIO TO MIN/MAX (0f/1f)
Audio.fu_Clamp = function (f_value, f_min, f_max) {
    return Math.max(f_min, Math.min(f_value, f_max));
}