﻿// GameRenderer =============================== //
// This class allows drawable objects to be         //
// registered and sorted according to depth before  //
// being rendered to the screen.                    //
// ============================================ DJS //
GameRenderer = function () {

}


// VARIABLES / OBJECTS ---------------------------- //
// ================================================ //
GameRenderer.sort = [];        // LIST OF DRAWABLE OBJECTS
GameRenderer.count = 0;        // NUMBER OF OBJECTS BEING DRAWN
GameRenderer.text = [];

// WII U OPTIMIZATION
GameRenderer.names = [];
GameRenderer.images = [];
GameRenderer.counts = [];
GameRenderer.srcRects = [];
GameRenderer.dstRects = [];
GameRenderer.mat32Buffer = [];
GameRenderer.alphaBuffer = [];

GameRenderer.arraysize = 310;


// INITIALIZE ===================================== //
// Initializes the array of drawable objects.       //
// ============================================ DJS //
GameRenderer.Initialize = function () {
    // INITALIZE THE ARRAY
    GameRenderer.sort = [];

    // INITIALIZE BUFFER ARRAYS FOR WII U
    if (GameCore.isWiiU)
        for (var i = 0; i < GameImages.i_textures; i++) {
            GameRenderer.srcRects[i] = GameRenderer.srcRects[i] || new Float32Array(4 * GameRenderer.arraysize);
            GameRenderer.dstRects[i] = GameRenderer.dstRects[i] || new Float32Array(4 * GameRenderer.arraysize);
            GameRenderer.mat32Buffer[i] = GameRenderer.mat32Buffer[i] || new Float32Array(6 * GameRenderer.arraysize);
            GameRenderer.alphaBuffer[i] = GameRenderer.alphaBuffer[i] || new Float32Array(4 * GameRenderer.arraysize);
        }
}


// REGISTER ======================================= //
// Adds an object to the list of drawable objects   //
// for sorting (and marks it as HUD).               //
// ============================================ DJS //
GameRenderer.fu_Register = function (obj) {
    // STORE THE OBJECT
    if (obj != undefined) {
        obj.true_depth = obj.depth + GameSceneManager.depth;
        GameRenderer.sort[GameRenderer.sort.length] = obj;
        GameRenderer.sort[GameRenderer.sort.length - 1].true_depth = GameSceneManager.depth + obj.depth;
    }
}

// DRAW =========================================== //
// Sorts the list of drawable objects and then      //
// draws them to the screen.                        //
// ============================================ DJS //
GameRenderer.Draw = function () {
    // SORT THE DRAWABLE OBJECTS - AT THE
    // MOMENT THIS USES A BUBBLE SORT, 
    // WHICH IS NOT IDEAL FOR LARGE LISTS
    // OF OBJECTS, SO THIS WILL NEED TO BE
    // REWRITTEN AT SOME POINT.  **TEMP**
    if (GameRenderer.sort.length > 1)
//        GameRenderer.sort = GameSort(GameRenderer.sort, (function (a, b) { return a.true_depth - b.true_depth; }));
// ORIGINAL SORT CODE
// INLINE BUBBLE SORT IS UNEQUIVOCALLY THE BETTER SORT ON WII U
        do
        {
            // ASSUME NOTHING SWAPS
            bSwapped = false;

            // BUBBLE SORT
            for (i = 1; i < GameRenderer.sort.length - 1; i++) {
                if (GameRenderer.sort[i - 1].true_depth > GameRenderer.sort[i].true_depth) {
                    var temp = GameRenderer.sort[i - 1];
                    GameRenderer.sort[i - 1] = GameRenderer.sort[i];
                    GameRenderer.sort[i] = temp;
                    bSwapped = true;
                }
            }
        } while (bSwapped);
       


    // WII U OPTIMIZATION TEST - COREY - REST OF FUNCTION IS TEST; ORIGINAL TWO LINES LABELED BELOW

    // DRAW THE OBJECTS
    if (GameCore.isWiiU) {

        // RESET DEPTH COMPARISON OPERATOR
        GameRenderer.depth_charge = -1;
        var i_sortLength = GameRenderer.sort.length;
		
        // LOOP THROUGH THE SORTED GAMESPRITES
        for (i = 0; i < i_sortLength; i++) {

            //		    if (!GameRenderer.sort[i].x_start || !GameRenderer.sort[i].y_start || !GameRenderer.sort[i].width || !GameRenderer.sort[i].height) 
            //		        console.log('name: ' + GameRenderer.sort[i].s_name + ' x start: ' + GameRenderer.sort[i].x_start + ' y start: ' + GameRenderer.sort[i].y_start + ' width: ' + GameRenderer.sort[i].width + ' height: ' + GameRenderer.sort[i].height);

//            if (GameRenderer.sort[i].text == null) {

                // SANITY CHECK THAT THE ITEM EXISTS AND IS LOADED
                if ((GameRenderer.sort[i].image != null && GameRenderer.sort[i].image.complete) || GameRenderer.sort[i].text != null) {

                    // NECESSARY TEXT TRUNCATION OF BUFFER CREATION IN ORDER TO MAINTAIN TEXT Z-ORDER
                    if (GameRenderer.sort[i].text != null) {

                        // SANITY CHECK ON FIRST PASS
                        if (GameRenderer.names && GameRenderer.names.length > 0) {

                            var i_namesLength = GameRenderer.names.length;

                            // EXECUTE DRAW PASS
                            for (var j = 0; j < i_namesLength; j++) {

                                // CHECK FOR OUT OF BOUNDS EXCEPTION
                                if (GameRenderer.counts[j] > 0 && GameRenderer.counts[j] < GameRenderer.arraysize) {

                                    // DRAW ALLOCATED INSTANCES FROM PREVIOUS DEPTH
                                    context.drawImageInstanced(GameRenderer.counts[j], GameRenderer.images[j], GameRenderer.srcRects[j], GameRenderer.dstRects[j], GameRenderer.mat32Buffer[j], GameRenderer.alphaBuffer[j]);

                                    // RESET MATRIX POSITION OF THE CONTEXT (NECESSARY AFTER DRAWIMAGEINSTANCED)
                                    context.setTransform(1, 0, 0, 1, 0, 0);

                                    // BUG HUNTING
                                } else {
                                    if (GameRenderer.counts[j] <= 0) {
                                        console.log('COUNTS LESS THAN 1');
                                    } else if (GameRenderer.counts[j] >= GameRenderer.arraysize) {
                                        console.log('COUNTS GREATER THAN ARRAY SIZE: ' + GameRenderer.counts[j]);
                                    }
                                }
                            }
                        }

                        GameRenderer.names = [];

                        // INITIALIZE VALUES
                        GameRenderer.counts[0] = 0;
                        GameRenderer.srcRects[0] = GameRenderer.srcRects[0] || new Float32Array(4 * GameRenderer.arraysize); // SHOULD NEVER BE MORE THAN 310? INSTANCES OF AN OBJECT IN A SINGLE ROW - IS THIS TOO LOW?
                        GameRenderer.dstRects[0] = GameRenderer.dstRects[0] || new Float32Array(4 * GameRenderer.arraysize);
                        GameRenderer.mat32Buffer[0] = GameRenderer.mat32Buffer[0] || new Float32Array(6 * GameRenderer.arraysize);
                        GameRenderer.alphaBuffer[0] = GameRenderer.alphaBuffer[0] || new Float32Array(4 * GameRenderer.arraysize);

                        // BUFFER THE IMAGE FOR DRAW
                        //GameRenderer.Buffer(0, GameRenderer.sort[i]);

                        // SET THE IMAGE PROPERTIES
                        //GameRenderer.images[0] = GameRenderer.sort[i].image;
                        //GameRenderer.names[0] = GameRenderer.sort[i].s_name;

                        //if (i >= i_sortLength - 1) {
                        //    context.drawImageInstanced(GameRenderer.counts[0], GameRenderer.images[0], GameRenderer.srcRects[0], GameRenderer.dstRects[0], GameRenderer.mat32Buffer[0], GameRenderer.alphaBuffer[0]);

                            // RESET MATRIX POSITION OF THE CONTEXT (NECESSARY AFTER DRAWIMAGEINSTANCED)
                        //    context.setTransform(1, 0, 0, 1, 0, 0);

                            // NULL DEPTH OPERATOR FOR NEXT CYCLE
                            GameRenderer.depth_charge = -1;

                        //    for (var k = GameRenderer.text.length; --k >= 0;)
                            GameRenderer.sort[i].SortedDraw();

                        //} else {

                            // SET DEPTH OPERATOR
                        //    GameRenderer.depth_charge = GameRenderer.sort[i].true_depth;
                        //}





                    } else 

                    // IF THE DEPTH OF THE SORTED ITEM IS DIFFERENT THAN THE CURRENT ROW OF INSTANCES
                    if (GameRenderer.sort[i].true_depth != GameRenderer.depth_charge) {

                        // SANITY CHECK ON FIRST PASS
                        if (GameRenderer.names && GameRenderer.names.length > 0) {

                            var i_namesLength = GameRenderer.names.length;

                            // EXECUTE DRAW PASS
                            for (var j = 0; j < i_namesLength; j++) {

                                // CHECK FOR OUT OF BOUNDS EXCEPTION
                                if (GameRenderer.counts[j] > 0 && GameRenderer.counts[j] < GameRenderer.arraysize) {

                                    // DRAW ALLOCATED INSTANCES FROM PREVIOUS DEPTH
                                    context.drawImageInstanced(GameRenderer.counts[j], GameRenderer.images[j], GameRenderer.srcRects[j], GameRenderer.dstRects[j], GameRenderer.mat32Buffer[j], GameRenderer.alphaBuffer[j]);

                                    // RESET MATRIX POSITION OF THE CONTEXT (NECESSARY AFTER DRAWIMAGEINSTANCED)
                                    context.setTransform(1, 0, 0, 1, 0, 0);

                                    // BUG HUNTING
                                } else {
                                    if (GameRenderer.counts[j] <= 0) {
                                        console.log('COUNTS LESS THAN 1');
                                    } else if (GameRenderer.counts[j] >= GameRenderer.arraysize) {
                                        console.log('COUNTS GREATER THAN ARRAY SIZE: ' + GameRenderer.counts[j]);
                                    }
                                }
                            }
                        }

                        GameRenderer.names = [];

                        // INITIALIZE VALUES
                        GameRenderer.counts[0] = 0;
                        GameRenderer.srcRects[0] = GameRenderer.srcRects[0] || new Float32Array(4 * GameRenderer.arraysize); // SHOULD NEVER BE MORE THAN 310? INSTANCES OF AN OBJECT IN A SINGLE ROW - IS THIS TOO LOW?
                        GameRenderer.dstRects[0] = GameRenderer.dstRects[0] || new Float32Array(4 * GameRenderer.arraysize);
                        GameRenderer.mat32Buffer[0] = GameRenderer.mat32Buffer[0] || new Float32Array(6 * GameRenderer.arraysize);
                        GameRenderer.alphaBuffer[0] = GameRenderer.alphaBuffer[0] || new Float32Array(4 * GameRenderer.arraysize);

                        // BUFFER THE IMAGE FOR DRAW
                        GameRenderer.Buffer(0, GameRenderer.sort[i]);

                        // SET THE IMAGE PROPERTIES
                        GameRenderer.images[0] = GameRenderer.sort[i].image;
                        GameRenderer.names[0] = GameRenderer.sort[i].s_name;

                        if (i >= i_sortLength - 1) {
                            context.drawImageInstanced(GameRenderer.counts[0], GameRenderer.images[0], GameRenderer.srcRects[0], GameRenderer.dstRects[0], GameRenderer.mat32Buffer[0], GameRenderer.alphaBuffer[0]);

                            // RESET MATRIX POSITION OF THE CONTEXT (NECESSARY AFTER DRAWIMAGEINSTANCED)
                            context.setTransform(1, 0, 0, 1, 0, 0);

                            // NULL DEPTH OPERATOR FOR NEXT CYCLE
                            GameRenderer.depth_charge = -1;

                            for (var k = GameRenderer.text.length; --k >= 0;)
                                GameRenderer.text[k].SortedDraw();

                        } else {

                            // SET DEPTH OPERATOR
                            GameRenderer.depth_charge = GameRenderer.sort[i].true_depth;
                        }

                        // IF THE DEPTH OF THE SORTED ITEM IS THE SAME AS THE CURRENT ROW OF INSTANCES
                    } else {

                        // IF THIS IS THE LAST ITEM IN THE SORTED LIST
                        if (i >= i_sortLength - 1) {

                            var i_namesLength = GameRenderer.names.length;

                            // EXECUTE DRAW PASS
                            for (var j = 0; j < i_namesLength; j++) {

                                // CHECK CURRENT NAME AGAINST NAMES IN THE DEPTH AND ASSIGN APPROPRIATELY
                                if (GameRenderer.sort[i].s_name == GameRenderer.names[j]) {

                                    // BUFFER THE IMAGE FOR DRAW
                                    GameRenderer.Buffer(j, GameRenderer.sort[i]);

                                }

                                // CHECK FOR OUT OF BOUNDS EXCEPTION
                                if (GameRenderer.counts[j] > 0) {

                                    // DRAW ALLOWCATED INSTANCES FROM LAST DEPTH
                                    context.drawImageInstanced(GameRenderer.counts[j], GameRenderer.images[j], GameRenderer.srcRects[j], GameRenderer.dstRects[j], GameRenderer.mat32Buffer[j], GameRenderer.alphaBuffer[j]);

                                    // RESET MATRIX POSITION OF THE CONTEXT (NECESSARY AFTER DRAWIMAGEINSTANCED)
                                    context.setTransform(1, 0, 0, 1, 0, 0);

                                }

                            }

                            // NULL DEPTH OPERATOR FOR NEXT CYCLE
                            GameRenderer.depth_charge = -1;

                            for (var k = GameRenderer.text.length; --k >= 0;)
                                GameRenderer.text[k].SortedDraw();

                            // IF THIS IS NOT THE LAST ITEM IN THE SORTED LIST
                        } else {

                            // GameControl VARIABLE FOR LOOP
                            var b_foundIt = false;
                            var i_namesLength = GameRenderer.names.length;

                            // COMPARE CURRENT ITEMS WITH ITEMS ALREADY AT CURRENT DEPTH
                            for (var j = 0; j < i_namesLength; j++) {

                                if (GameRenderer.sort[i].s_name == GameRenderer.names[j]) {

                                    // BUFFER THE IMAGE FOR DRAW
                                    GameRenderer.Buffer(j, GameRenderer.sort[i]);

                                    b_foundIt = true;

                                    break;
                                }
                            }

                            // IF AN INSTANCE OF THE CURRENT ITEM DOES NOT ALREADY EXIST AT THIS DEPTH
                            if (!b_foundIt) {

                                // INITIALIZE I AM SO TIRED OF COMMENTING THIS KLUGE
                                GameRenderer.counts[i_namesLength] = 0;
                                GameRenderer.srcRects[i_namesLength] = GameRenderer.srcRects[i_namesLength] || new Float32Array(4 * GameRenderer.arraysize);
                                GameRenderer.dstRects[i_namesLength] = GameRenderer.dstRects[i_namesLength] || new Float32Array(4 * GameRenderer.arraysize);
                                GameRenderer.mat32Buffer[i_namesLength] = GameRenderer.mat32Buffer[i_namesLength] || new Float32Array(6 * GameRenderer.arraysize);
                                GameRenderer.alphaBuffer[i_namesLength] = GameRenderer.alphaBuffer[i_namesLength] || new Float32Array(4 * GameRenderer.arraysize);

                                // BUFFER THE IMAGE FOR DRAW
                                GameRenderer.Buffer(i_namesLength, GameRenderer.sort[i]);

                                // SET PROPERTIES OF THE IMAGE
                                GameRenderer.images[i_namesLength] = GameRenderer.sort[i].image;
                                GameRenderer.names[i_namesLength] = GameRenderer.sort[i].s_name;
                            }
                        }
                    }
                }
//            } else {
//                if (GameRenderer.sort[i].text != null)
//                    GameRenderer.text.push(GameRenderer.sort[i]);
//            }
        }
    } else {
        // EVERYTHING EXCEPT WII U
        for (i = 0; i < GameRenderer.sort.length; i++)
            GameRenderer.sort[i].SortedDraw();
    }
        
        // WII U OPTIMIZATION - COREY - ORIGINAL - UNCOMMENT NEXT TWO LINES!!!!!!
        //for (i = 0; i < GameRenderer.sort.length; i++)
		//	GameRenderer.sort[i].SortedDraw();

    // RECORD SPRITE COUNT, IF IT HAS CHANGED
/*    if (GameRenderer.count != GameRenderer.sort.length) {
          GameRenderer.count = GameRenderer.sort.length;
          GameLog.Record("Sorter | " + GameRenderer.sort.length + " Drawable Objects");
      }
*/
}

GameRenderer.Buffer = function (i_arrayPos, o_sorted) {

    // INDEX BUFFER ARRAY GameControl
    var i_indexFour = GameRenderer.counts[i_arrayPos] * 4;
    var i_indexSix = GameRenderer.counts[i_arrayPos] * 6;

    var angle_sine = 0;
    var angle_cosine = 1;

    if (o_sorted.rotation) {
        //var i_rotationRad = GameVector.GetRadFromDeg(o_sorted.rotation);
        angle_sine = GameHelper.sin(o_sorted.rotation)
        angle_cosine = GameHelper.cos(o_sorted.rotation);
    }

    // SET SOURCE RECTANGLES
    GameRenderer.srcRects[i_arrayPos][i_indexFour + 0] = o_sorted.x_start;
    GameRenderer.srcRects[i_arrayPos][i_indexFour + 1] = o_sorted.y_start;
    GameRenderer.srcRects[i_arrayPos][i_indexFour + 2] = o_sorted.width;
    GameRenderer.srcRects[i_arrayPos][i_indexFour + 3] = o_sorted.height;

    if (o_sorted.bIsHUD) {
        // SET DESTINATION RECTANGLES - DON'T SCALE FOR HUD
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 0] = -(o_sorted.width * o_sorted.origin.i_x);
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 1] = -(o_sorted.height * o_sorted.origin.i_y);
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 2] = o_sorted.width;
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 3] = o_sorted.height;

        // SET TRANFORMATION MATRIX OF DEST RECT - DON'T SCALE FOR HUD - SCALE X, SKEW X, SKEW Y, SCALE Y, TRANSLATE X, TRANSLATE Y
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 0] = (o_sorted.scale ? (o_sorted.flip_horz ? o_sorted.scale.i_x * -1 : o_sorted.scale.i_x) : 1);
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 1] = 0;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 2] = 0;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 3] = (o_sorted.scale ? (o_sorted.flip_vert ? o_sorted.scale.i_y * -1 : o_sorted.scale.i_y) : 1);
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 4] = o_sorted.position.X;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 5] = o_sorted.position.Y;

        // ALPHA AND COLOR VALUES - ALPHA DEFAULTS TO 1 FOR HUD - R, G, B, A
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 0] = 1;
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 1] = 1;
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 2] = 1;
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 3] = ((o_sorted.alpha >= 0) ? o_sorted.alpha : 1);
    } else {
        // SET DESTINATION RECTANGLES - SCALE FOR NON-HUD
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 0] = -(o_sorted.width * o_sorted.origin.i_x * GameHelper.GlobalScale);
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 1] = -(o_sorted.height * o_sorted.origin.i_y * GameHelper.GlobalScale);
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 2] = o_sorted.width * GameHelper.GlobalScale;
        GameRenderer.dstRects[i_arrayPos][i_indexFour + 3] = o_sorted.height * GameHelper.GlobalScale;

        // SET TRANFORMATION MATRIX OF DEST RECT - SCALE FOR NON-HUD - SCALE X, SKEW X, SKEW Y, SCALE Y, TRANSLATE X, TRANSLATE Y
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 0] = (o_sorted.scale ? (o_sorted.flip_horz ? o_sorted.scale.i_x * -1 : o_sorted.scale.i_x) : 1) * angle_cosine;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 1] = angle_sine;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 2] = -angle_sine;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 3] = (o_sorted.scale ? (o_sorted.flip_vert ? o_sorted.scale.i_y * -1 : o_sorted.scale.i_y) : 1) * angle_cosine;
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 4] = (o_sorted.position.X * GameHelper.GlobalScale - GameCamera.position.X);
        GameRenderer.mat32Buffer[i_arrayPos][i_indexSix + 5] = (o_sorted.position.Y * GameHelper.GlobalScale - GameCamera.position.Y);

        // ALPHA AND COLOR VALUES - ALPHA DEFAULTS TO 0 FOR NON-HUD - R, G, B, A
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 0] = 1;
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 1] = 1;
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 2] = 1;
        GameRenderer.alphaBuffer[i_arrayPos][i_indexFour + 3] = (o_sorted.alpha ? o_sorted.alpha : 0);
    }

    // INCREMENT COUNT
    GameRenderer.counts[i_arrayPos]++;

}


GameRenderer.Clear = function () {
    // CLEAR THE ARRAY
    GameRenderer.sort = [];
    GameRenderer.text = [];
}


// .                                                //
// .                                                //
// .                                                //
// CHANGE LOG ===================================== //
//                                                  //
// Oct. 14th, 2014 (DJS) -------------------------- //
//      * Added support for true_depth.             //
//                                                  //
// Dec. 12th, 2013 (DJS) -------------------------- //
//      * Added a clear method.                     //
//                                                  //
// Dec. 1st, 2013 (DJS) --------------------------- //
//      * Created the file.                         //
//                                                  //