﻿// GAMESPRITE ===================================== //
// Represents a sprite in the application.			//
// ============================================ DJS //

function GameSprite() {

    // INHERETED FROM DRAWABLE OBJECT
    this.camera_effect = 1;     // HOW MUCH DOES THE CAMERA IMPACT POSITION?

    // VARIABLES / OBJECTS ---------------------------- //
    // ================================================ //

    // THE IMAGE TO DRAW 
    this.image = null;

    // THE IMAGE EXTENSION
    this.extension = "png";

    // IDENTIFYING INFORMATION
    this.iID = -1;
    this.name = null;

    // TEMP WII U TEST
    this.x_start = null;
    this.y_start = null;
    this.width = null;
    this.height = null;
    this.x_end = null;
    this.y_end = null;
    this.s_name = null;

    // PRESENTATION PARAMETERS
    this.position = new GameVec(0, 0);
    this.origin = new GameVec(0.5, 0.5);
    this.rotation = 0;
    this.scale = new GameVec(1, 1);
    this.alpha = 1;

    // KLUGE
    this.skip_onloaded = false;

    //
    this.flip_horz = false;
    this.flip_vert = false;


    // INITIALIZE ===================================== //
    // Takes one argument: the name of the .PNG file    //
    // to load.											//
    // ============================================ DJS //
    this.fu_Initialize = function (name, position, origin, rotation, scale, isURL) {

        // SANITY CHECK
        if (name == null)
            return;

        // CHECK THAT THE RESOURCE EXISTS
        ;

        // STORE NAME
        this.name = name;

        if (isURL == null)
            isURL = false;

            // INCREASE LOAD COUNT
            if (!this.skip_onloaded)
                GameLoop.i_loading++;
            // LOAD THE IMAGE
            this.image = new Image();

            // INCREASE LOADED COUNT WHEN THE SPRITE LOADS
            if (!this.skip_onloaded) {
                var me = this;
                this.x_start = 0;
                this.y_start = 0;
                this.image.onload = function () {
                    GameLoop.i_loaded++;
                    me.width = this.width;
                    me.height = this.height;

                    if (GameConstants.B_DEBUG) { console.log('IMAGE LOADED: ' + name); }

                    GameLoop.fu_ContentCheck();
                }
            }

            if (!isURL) {
            
                // SET SPRITE
                this.image.src = "Resources/Textures/" + name + "." + this.extension;

            } else {

                // SET SPRITE
                this.image.src = name;
            }


        // SET INITIAL POSITION
        if (position != null)
            this.position = position;

        // SET ORIGIN
        if (origin != null)
            this.origin = origin;

        // SET ROTATION
        if (rotation != null)
            this.rotation = rotation;

        // SET SCALE
        if (scale != null)
            this.scale = scale;

        // SECURE ID
        this.iID = GameSprite.next_id++;
    }


    // UPDATE ========================================= //
    // Blank at the moment - might be overridden by		//
    // child classes at some point.						//
    // ============================================ DJS //
    this.fu_Update = function () {

       
    }

    // DRAW =========================================== //
    // This method registers the GameSprite as a        //
    // drawable object.                                 //
    // ============================================ DJS //
    this.fu_Draw = function () {
        // SAVE THE CANVAS
        context.save();

        // SET GLOBAL ALPHA FOR TRANSPARENCY EFFECTS
        if (context.globalAlpha != this.alpha)
            context.globalAlpha = this.alpha;

        var x = Math.floor(this.position.i_x);
        var y = Math.floor(this.position.i_y);
      
        // TRANSFORM THE CANVAS
        context.translate(x, y);

        if (this.rotation != 0)
            context.rotate(this.rotation);

        if (this.scale.i_x != 1 || this.scale.i_y != 1 || this.flip_horz || this.flip_vert)
            context.scale(this.flip_horz ? this.scale.i_x * -1 : this.scale.i_x, this.flip_vert ? this.scale.i_y * -1 : this.scale.i_y);

        // DRAW THE IMAGE
        if (this.image != null)
            if (this.image.complete)
                context.drawImage(this.image, this.x_start, this.y_start, this.width, this.height, -(this.width * this.origin.i_x), -(this.height * this.origin.i_y), this.width, this.height);
                
        // RESET THE CANVAS
        context.restore();
    }

}


// STATIC VARIABLES / OBJECTS --------------------- //
// ================================================ //
GameSprite.next_id = 0;       		// GIVE EACH GAMESPRITE A UNIQUE ID



// .                                                //
// .                                                //
// .                                                //
// CHANGE LOG ===================================== //
//                                                  //
// Jul. 5th, 2014 (DJS) --------------------------- //
//      * Tidied up.                                //
//      * Removed variables for tracking loads.     //
//                                                  //
// Jun. 25th, 2014 (DJS) -------------------------- //
//		* Added DrawWhole technique to SortedDraw.	//
//													//
// Dec. 12th, 2013 (DJS) -------------------------- //
//      * Added a Clone method.  Might not be       //
//        necessary, but oh well.                   //
//      * Added a global count and individual IDs.  //
//                                                  //
// Dec. 10th, 2013 (DJS) -------------------------- //
//      * Integrated a camera effect for parallax.  //
//                                                  //
// Dec. 1st, 2013 (DJS) --------------------------- //
//      * Rigged GameSprite as a drawable object,   //
//        so that it's linked to the GameSprite     //
//        sorter system.                            //
//                                                  //
// Nov. 30th, 2013 (DJS) -------------------------- //
//		* Added HUD support to bypass the camera.	//
//		* Added transparency support.				//
//													//
// Nov. 28th, 2013 (DJS) -------------------------- //
//      * Moved functionality from constructor to   //
//		  the Initialize method.					//
//		* Added sanity checks to prevent errors.	//
//                                                  //