﻿// GAMETEXTALIGNMENT ============================== //
// This enum represents the three ways in which		//
// GameText may be aligned: left, center, right.	//
// ============================================ DJS //
GameTextAlignment =
{
    LEFT: 0,
    CENTER: 1,
    RIGHT: 2
}


// GAMETEXT ======================================= //
// This class represents text that appears on the	//
// screen.  It may be HUD text, free from the		//
// influence of the GameCamera, or otherwise.		//
// ============================================ DJS //
function GameText() {

    // VARIABLES / OBJECTS ---------------------------- //
    // ================================================ //
    this.text = null;							// TEXT TO DISPLAY
    this.position = null;						// POSITION OF THE TEXT
    this.color = GameColor.Clone(GameColor.WHITE);				// TEXT COLOR
    this.font = "Arial";						// TEXT FONT
    this.size = 12;								// TEXT SIZE
    this.alignment = GameTextAlignment.CENTER;	// TEXT ALIGNMENT
    this.alpha = 1;
	this.multiline = false;						// DOES THIS TEXT OVERFLOW?
	this.max_width = 256;						// WIDTH ALLOWED BEFORE OVERFLOW
	this.line_space = 12;                       // DEFAULT SPACE BETWEEN LINES
	this.style = "";
	this.lines = 1;

    // DRAW =========================================== //
    // This method registers the GameText as a          //
    // drawable object.                                 //
    // ============================================ DJS //
    this.fu_Draw = function () {
        // PREP THE CANVAS
        this.PrepCanvas();

        
			switch (this.alignment) {
				case GameTextAlignment.LEFT:
					context.fillText(this.text, Math.floor(this.position.i_x), Math.floor(this.position.i_y));
					break;
				case GameTextAlignment.CENTER:
					context.fillText(this.text, Math.floor(this.position.i_x - this.GetWidth() / 2.0), Math.floor(this.position.i_y));
					break;
				case GameTextAlignment.RIGHT:
					context.fillText(this.text, Math.floor(this.position.i_x - this.GetWidth()), Math.floor(this.position.i_y));
					break;
			}
    }

	
    // CLEANUP ======================================== //
    // This method gets rid of unused resources.		//
    // ============================================ DJS //
    this.CleanUp = function () {

    }


    // PREPCANVAS ===================================== //
    // Sets the font and style onto the canvas.			//
    // ============================================ DJS //
    this.PrepCanvas = function () {

        //
        context.globalAlpha = this.alpha;
        // SET COLOR
        context.fillStyle = this.color.ToRGB();

        // SET FONT
        context.font = this.style + " " + this.size + "px '" + this.font + "'";
    }


    // GETWIDTH ======================================= //
    // Returns the pixel width of the text.				//
    // ============================================ DJS //
    this.GetWidth = function () {
        // SANITY CHECK
        if (this.text == null)
            return 0;

        // PREP THE CANVAS
        this.PrepCanvas();

        // RETURN THE WIDTH
        return context.measureText(this.text).width;
    }


    // GETHEIGHT ====================================== //
    // Returns the pixel height of the text.			//
    // ============================================ DJS //
    this.GetHeight = function () {
        // SANITY CHECK
        if (this.text == null)
            return 0;

        // PREP THE CANVAS
        this.PrepCanvas();

        // RETURN THE WIDTH
        return (this.size / 72.0) * 96.0; // **TEMP**
    }
	
}


// .                                                //
// .                                                //
// .                                                //
// CHANGE LOG ===================================== //
//                                                  //
// Jun. 25th, 2014 (DJS) -------------------------- //
//		* Added DrawWhole technique to SortedDraw.	//
//													//
// Dec. 1st, 2013 (DJS) --------------------------- //
//      * Rigged GameText as a drawable object, so  //
//        that it's linked to the GameSprite        //
//        sorter system.                            //
//                                                  //
// Nov. 30th, 2013 (DJS) -------------------------- //
//      * Created the file.                         //
//                                                  //